@@ -3,7 +3,7 @@
 # Programmer friendly subprocess wrapper.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: November 10, 2015
+# Last Change: November 13, 2015
 # URL: https://executor.readthedocs.org
 
 """
@@ -64,7 +64,7 @@ except NameError:
     unicode = str
 
 # Semi-standard module versioning.
-__version__ = '7.7'
+__version__ = '8.0'
 
 # Initialize a logger.
 logger = logging.getLogger(__name__)
@@ -538,6 +538,10 @@ class ExternalCommand(ControllableProcess):
     features (e.g. asynchronous command execution that preserves the ability to
     provide input and capture output).
 
+    :class:`ExternalCommand` inherits from :class:`ControllableProcess` so all
+    of the process manipulation supported by :class:`ControllableProcess` is
+    also supported by :class:`ExternalCommand` objects.
+
     Because the :class:`ExternalCommand` class has a lot of properties and
     methods here is a summary:
 
@@ -560,9 +564,9 @@ class ExternalCommand(ControllableProcess):
      and what its output is.
 
     **Public methods**
-     The public methods :func:`start()`, :func:`wait()` and :func:`terminate()`
-     enable you to start external commands, wait for them to finish and
-     terminate them if they take too long.
+     The public methods :func:`start()`, :func:`wait()`, :func:`terminate()`
+     and :func:`kill()` enable you to start external commands, wait for them to
+     finish and terminate them if they take too long.
 
     **Internal methods**
      The internal methods :func:`check_errors()`, :func:`load_output()` and
